# -*- coding: utf-8 -*-

"""Includes functions for manipulating strings."""

import os


def generate_folder_name_and_path(name, cwd):
    """
    Returns a folder name and path based on the project name and current working directory.

    :param name: Project name
    :param cwd: Current working directory
    :return: New folder name and path
    """
    folder_name = name.lower().replace(" ", "_")
    folder_path = os.path.join(cwd, folder_name)
    return folder_name, folder_path



# -*- coding: utf-8 -*-

"""Includes test for the 'pynx' command"""

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import assert_readme_has_project_and_author_name, assert_file_contains_strings, \
    directory_exists, APP_NAME, APP_AUTHOR, DATE_CREATED


def test_pynx():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['-n', APP_NAME, '-a', APP_AUTHOR, 'pynx'])
        assert not result.exception
        assert result.output.endswith('Successfully created the PyNX project!\n')
        assert directory_exists()
        assert_file_contains_strings('main.py', [APP_NAME, APP_AUTHOR, DATE_CREATED])
        assert_readme_has_project_and_author_name()



# -*- coding: utf-8 -*-

"""Contains helper functions to use in tests"""

import datetime
import os

# Some constants to be used in tests
APP_NAME = 'Test project'
APP_AUTHOR = 'Ruud Schroën'
DATE_CREATED = datetime.datetime.now().strftime("%Y-%m-%d")
DIRECTORY_NAME = 'test_project'


def directory_exists():
    """
    Check if a folder called DIRECTORY_NAME exists
    """
    return os.path.isdir(DIRECTORY_NAME)


def file_exists(file_path):
    """
    Check if the file at file_path exists.

    :param file_path: File to check
    """
    return os.path.isfile(os.path.join(DIRECTORY_NAME, file_path))


def assert_file_contains_strings(file_path, strings):
    """
    Make sure that each given string is in the file.

    :param file_path: File to check
    :param strings: List of strings
    """
    with open(os.path.join(DIRECTORY_NAME, file_path), 'r') as file:
        data = file.read().replace('\n', '')
        for s in strings:
            assert s in data


def assert_readme_has_project_and_author_name():
    """
    Checks that the README.md contains the project and author name.
    """
    assert_file_contains_strings('README.md', [APP_NAME, APP_AUTHOR])


def assert_makefile_has_project_and_author_name():
    """
    Check that the Makefile contains the project and author name.
    """
    assert_file_contains_strings('Makefile', [APP_NAME, APP_AUTHOR])



"""
    Main file for APP_NAME_PLACEHOLDER

    File name: main.py
    Author: APP_AUTHOR_PLACEHOLDER
    Date created: DATE_PLACEHOLDER
    Date last modified: DATE_PLACEHOLDER
"""

import nx


def main():
    """
    The main loop
    """
    print('Hello world!')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""Defines the app context and commands."""

import os

import click

from nxstart import app


class Context(object):
    """
    Context to be passed to the subcommands.
    """

    def __init__(self):
        self.name = None
        self.author = None
        self.cwd = os.getcwd()


pass_context = click.make_pass_decorator(Context, ensure=True)


@click.group()
@click.option('--name', '-n', default=None, prompt='Project name', help='The name of your project')
@click.option('--author', '-a', default=None, prompt='Author name',
              help='The full name of the author')
@pass_context
def cli(ctx, name, author):
    ctx.name = name
    ctx.author = author


@cli.command('libnx', short_help='create a new libnx project (C++)')
@click.option('--clion/--no-clion', default=False, prompt='Are you using CLion?', help='include CMakeLists.txt')
@pass_context
def libnx(ctx, clion):
    """
    Command for generating a libnx project.

    :param ctx: Context
    :param clion: Using CLion
    """
    app.libnx(ctx.name, ctx.author, clion, ctx.cwd)


@cli.command('libt', short_help='create a new libtransistor project (C)')
@click.option('--clion/--no-clion', default=False, prompt='Are you using CLion?', help='include CMakeLists.txt')
@pass_context
def libt(ctx, clion):
    """
    Command for generating a libtransistor project.

    :param ctx: Context
    :param clion: Using CLion
    """
    app.libt(ctx.name, ctx.author, clion, ctx.cwd)


@cli.command('brewjs', short_help='create a new BrewJS project (Javascript)')
@pass_context
def brewjs(ctx):
    """
    Command for generating a BrewJS project.

    :param ctx: Context
    """
    app.brewjs(ctx.name, ctx.author, ctx.cwd)


@cli.command('pynx', short_help='create a new PyNX project (Python)')
@pass_context
def pynx(ctx):
    """
    Command for generating a PyNX project.

    :param ctx: Context
    """
    app.pynx(ctx.name, ctx.author, ctx.cwd)



# -*- coding: utf-8 -*-

"""Includes tests for the 'libnx' command"""

import os

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import directory_exists, assert_readme_has_project_and_author_name, \
    assert_makefile_has_project_and_author_name, assert_file_contains_strings, file_exists, APP_AUTHOR, APP_NAME, \
    DATE_CREATED


def test_libnx_with_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['-n', APP_NAME, '-a', APP_AUTHOR, 'libnx', '--clion'])
        assert not result.exception
        assert result.output.endswith('Successfully created the libnx project!\n')
        assert directory_exists()
        assert file_exists('CMakeLists.txt')
        assert_readme_has_project_and_author_name()
        assert_makefile_has_project_and_author_name()
        main_cpp_has_valid_data()


def test_libnx_without_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['-n', APP_NAME, '-a', APP_AUTHOR, 'libnx', '--no-clion'])
        assert not result.exception
        assert result.output.endswith('Successfully created the libnx project!\n')
        assert directory_exists()
        assert not file_exists('CMakeLists.txt')
        assert_readme_has_project_and_author_name()
        assert_makefile_has_project_and_author_name()
        main_cpp_has_valid_data()


def main_cpp_has_valid_data():
    assert_file_contains_strings(os.path.join('source', 'main.cpp'), [APP_NAME, APP_AUTHOR, DATE_CREATED])



# -*- coding: utf-8 -*-

"""Includes generic functions such as copying the README.md file."""

import os

from nxstart.utils.files import replace_in_file


def modify_readme_file(folder_path, name, author):
    """
    Simple helper for modifying README.md files.

    :param folder_path: Path where the README.md file is
    :param name: Project name
    :param author: Project author
    """
    new_readme_file = os.path.join(folder_path, 'README.md')
    new_readme_file_replacements = {
        'APP_NAME_PLACEHOLDER': name,
        'APP_AUTHOR_PLACEHOLDER': author

    }
    replace_in_file(new_readme_file, new_readme_file_replacements)


def remove_cmake_lists_file(folder_path):
    """
    Removes the CMakeLists.txt file inside the folder at folder_path.

    :param folder_path: Path to created folder
    """
    cmake_lists_file = os.path.join(folder_path, 'CMakeLists.txt')
    os.remove(cmake_lists_file)


def modify_cmake_lists_file(folder_path, folder_name):
    """
    Modifies the CMakeLists.txt file from folder_path, and will use folder_name as the project name.

    :param folder_path: Path to created folder
    :param folder_name: Project folder name
    """
    cmake_lists_file = os.path.join(folder_path, 'CMakeLists.txt')
    cmake_lists_file_replacements = {
        'FOLDER_NAME_PLACEHOLDER': folder_name
    }
    replace_in_file(cmake_lists_file, cmake_lists_file_replacements)



# -*- coding: utf-8 -*-

"""Includes tests for the functions in nxstart.utils.strings"""

import os

from nxstart.tests.helpers import DIRECTORY_NAME, APP_NAME
from nxstart.utils.strings import generate_folder_name_and_path


def test_generate_folder_name_and_path():
    cwd = os.getcwd()
    folder_name, folder_path = generate_folder_name_and_path(APP_NAME, cwd)
    assert folder_name == DIRECTORY_NAME
    assert folder_path == os.path.join(cwd, folder_name)



# -*- coding: utf-8 -*-

"""Includes test for the 'brewjs' command"""

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import directory_exists, assert_file_contains_strings, APP_NAME, APP_AUTHOR, DATE_CREATED


def test_brewjs():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ['-n', APP_NAME, '-a', APP_AUTHOR, 'brewjs'])
        assert not result.exception
        assert result.output.endswith('Successfully created the BrewJS project!\n')
        assert directory_exists()
        assert_file_contains_strings('index.js', [APP_NAME, APP_AUTHOR, DATE_CREATED])



# -*- coding: utf-8 -*-

"""Includes tests for the functions in nxstart.utils.files"""

import os

from nxstart.tests.helpers import DIRECTORY_NAME, assert_file_contains_strings
from nxstart.utils.files import check_and_create_directory, PROJECT_ROOT, get_full_path, replace_in_file


def test_get_full_path():
    path = os.path.join(PROJECT_ROOT, 'test.py')
    path_two = get_full_path('test.py')
    assert path == path_two


def test_check_and_create_directory(tmpdir):
    new_folder_path = str(tmpdir.join(DIRECTORY_NAME))

    check_and_create_directory(new_folder_path)
    assert os.path.isdir(new_folder_path)


def test_replace_in_file(tmpdir):
    new_test_file_path = str(tmpdir.join('testfile.txt'))
    with open(new_test_file_path, 'w') as f:
        f.write('TEXT_PLACEHOLDER')
    replace_in_file(new_test_file_path, {
        'TEXT_PLACEHOLDER': 'NEW_TEXT',
    })
    assert_file_contains_strings(new_test_file_path, ['NEW_TEXT'])




# -*- coding: utf-8 -*-

"""Includes functions for copying the libtransistor template files."""

import datetime
import os
from distutils.dir_util import copy_tree

from nxstart.utils.files import get_full_path, replace_in_file


def create_libt_project(folder_path, name, author):
    """
    Copies the files from templates/base to folder_path and modifies Makefile and source/main.cpp
    to include the project name, author name and current date.

    :param folder_name: Created folder name
    :param folder_path: Path to copy the files to
    :param name: Name of the project
    :param author: Name of the author
    """
    template_folder = get_full_path(os.path.join('templates', 'libt'))
    copy_tree(template_folder, folder_path)

    main_c_file = os.path.join(folder_path, 'main.c')
    main_c_replacements = {
        'APP_AUTHOR_PLACEHOLDER': author,
        'APP_NAME_PLACEHOLDER': name,
        'DATE_PLACEHOLDER': datetime.datetime.now().strftime("%Y-%m-%d")
    }
    replace_in_file(main_c_file, main_c_replacements)

    makefile = os.path.join(folder_path, 'Makefile')
    makefile_replacements = {
        'APP_NAME_PLACEHOLDER': name,
        'APP_AUTHOR_PLACEHOLDER': author
    }
    replace_in_file(makefile, makefile_replacements)



# -*- coding: utf-8 -*-

"""Defines the logic for the commands."""

import click

from nxstart import filebuilder
from nxstart.utils.files import check_and_create_directory
from nxstart.utils.strings import generate_folder_name_and_path


def libnx(name, author, clion, cwd):
    """
    Function that holds the logic for the 'libnx' command.

    :param name: Name of the project
    :param author: Name of the author
    :param clion: Using CLion
    :param cwd: Current working directory
    """
    folder_name, folder_path = generate_folder_name_and_path(name, cwd)
    check_and_create_directory(folder_path)

    filebuilder.libnx.create_libnx_project(folder_path, name, author)
    filebuilder.generic.modify_readme_file(folder_path, name, author)

    if clion:
        filebuilder.generic.modify_cmake_lists_file(folder_path, folder_name)
    else:
        filebuilder.generic.remove_cmake_lists_file(folder_path)

    click.echo("Successfully created the libnx project!")


def libt(name, author, clion, cwd):
    """
    Function that holds the logic for the 'libt' command.

    :param name: Name of the project
    :param author: Name of the author
    :param clion: Using CLion
    :param cwd: Current working directory
    """
    folder_name, folder_path = generate_folder_name_and_path(name, cwd)
    check_and_create_directory(folder_path)

    filebuilder.libt.create_libt_project(folder_path, name, author)
    filebuilder.generic.modify_readme_file(folder_path, name, author)

    if clion:
        filebuilder.generic.modify_cmake_lists_file(folder_path, folder_name)
    else:
        filebuilder.generic.remove_cmake_lists_file(folder_path)

    click.echo("Successfully created the libtransistor project!")


def brewjs(name, author, cwd):
    """
    Function that holds the logic for the 'brewjs' command.

    :param name: Name of the project
    :param author: Name of the author
    :param cwd: Current working directory
    """
    folder_name, folder_path = generate_folder_name_and_path(name, cwd)
    check_and_create_directory(folder_path)

    filebuilder.brewjs.create_brewjs_project(folder_path, name, author)
    filebuilder.generic.modify_readme_file(folder_path, name, author)

    click.echo("Successfully created the BrewJS project!")


def pynx(name, author, cwd):
    """
    Function that holds the logic for the 'pynx' command.

    :param name: Name of the project
    :param author: Name of the author
    :param cwd: Current working directory
    """
    folder_name, folder_path = generate_folder_name_and_path(name, cwd)
    check_and_create_directory(folder_path)

    filebuilder.pynx.create_pynx_project(folder_path, name, author)
    filebuilder.generic.modify_readme_file(folder_path, name, author)

    click.echo("Successfully created the PyNX project!")
